'use strict';

exports.__esModule = true;
exports["default"] = undefined;

var _react = require('react');

var _storeShape = require('../utils/storeShape');

var _storeShape2 = _interopRequireDefault(_storeShape);

var _warning = require('../utils/warning');

var _warning2 = _interopRequireDefault(_warning);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var didWarnAboutReceivingStore = false;
function warnAboutReceivingStore() {
  if (didWarnAboutReceivingStore) {
    return;
  }
  didWarnAboutReceivingStore = true;

  (0, _warning2["default"])('<Provider> does not support changing `store` on the fly. ' + 'You are most likely seeing this error because you updated to ' + 'Redux 2.x and React Redux 2.x which no longer hot reload reducers ' + 'automatically. See https://github.com/reactjs/react-redux/releases/' + 'tag/v2.0.0 for the migration instructions.');
}

var Provider = function (_Component) {
  _inherits(Provider, _Component);

  Provider.prototype.nextUniqueId = function nextUniqueId() {
    var currentUniqueId = Number(this.currentUniqueId);
    ++this.currentUniqueId;
    return currentUniqueId;
  };

  Provider.prototype.getChildContext = function getChildContext() {
    var _this2 = this;

    return {
      store: this.store,
      getUniqueId: this.nextUniqueId.bind(this),
      setMapStateToProps: this.setMapStateToProps,
      unsetMapStateToProps: this.unsetMapStateToProps,
      getBlackboxFacsimiles: function getBlackboxFacsimiles(id) {
        return _this2.blackboxFacsimiles[id];
      }
    };
  };

  function Provider(props, context) {
    _classCallCheck(this, Provider);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props, context));

    _this.store = props.store;
    _this.currentUniqueId = 1;
    _this.blackbox = {};
    _this.blackboxFacsimiles = {};
    _this.allMapStateToProps = {};
    _this.getBlackbox = _this.getBlackbox.bind(_this);
    _this.setMapStateToProps = _this.setMapStateToProps.bind(_this);
    _this.unsetMapStateToProps = _this.unsetMapStateToProps.bind(_this);
    var storeState = _this.store.getState();
    _this.state = { storeState: storeState };
    return _this;
  }

  Provider.prototype.render = function render() {
    var blackbox = this.getBlackbox();
    if (this.props.children === undefined || Array.isArray(this.props.children)) {
      return _react.Children.only(this.props.children);
    }
    return (0, _react.cloneElement)(this.props.children, { blackbox: blackbox });
  };

  Provider.prototype.getBlackbox = function getBlackbox() {
    var _this3 = this;

    var nextBlackbox = {};
    this.blackboxFacsimiles = {};
    Object.keys(this.allMapStateToProps).forEach(function (id) {
      var idSelector = _this3.allMapStateToProps[id];

      var _idSelector = idSelector(_this3.state.storeState);

      var change = _idSelector[0];
      var nextStateProps = _idSelector[1];


      _this3.blackboxFacsimiles[id] = !change;
      nextBlackbox[id] = nextStateProps;
    });
    this.blackbox = nextBlackbox;
    return this.blackbox;
  };

  Provider.prototype.setMapStateToProps = function setMapStateToProps(id, selector) {
    this.allMapStateToProps[id] = selector;
  };

  Provider.prototype.unsetMapStateToProps = function unsetMapStateToProps(id) {
    delete this.allMapStateToProps[id];
  };

  Provider.prototype.trySubscribe = function trySubscribe(c) {
    c.unsubscribe = c.store.subscribe(c.handleChange.bind(c));
    c.handleChange();
  };

  Provider.prototype.getPure = function getPure(c) {
    if (c.props.pure !== undefined) {
      return c.props.pure;
    }
    return true;
  };

  Provider.prototype.handleChange = function handleChange() {
    if (!this.unsubscribe) {
      return;
    }

    var pure = this.getPure(this);
    var storeState = this.store.getState();
    var prevStoreState = this.state.storeState;
    if (pure && prevStoreState === storeState) {
      return;
    }
    this.setState({ storeState: storeState });
  };

  Provider.prototype.tryUnsubscribe = function tryUnsubscribe(c) {
    if (c.unsubscribe) {
      c.unsubscribe();
      c.unsubscribe = null;
    }
  };

  Provider.prototype.componentDidMount = function componentDidMount() {
    this.trySubscribe(this);
  };

  Provider.prototype.componentWillUnmount = function componentWillUnmount() {
    this.tryUnsubscribe(this);
  };

  return Provider;
}(_react.Component);

exports["default"] = Provider;


if (process.env.NODE_ENV !== 'production') {
  Provider.prototype.componentWillReceiveProps = function (nextProps) {
    var store = this.store;
    var nextStore = nextProps.store;


    if (store !== nextStore) {
      warnAboutReceivingStore();
      this.trySubscribe(this);
    }
  };
}

Provider.propTypes = {
  store: _storeShape2["default"].isRequired,
  children: _react.PropTypes.element.isRequired
};
Provider.childContextTypes = {
  store: _storeShape2["default"].isRequired,
  getUniqueId: _react.PropTypes.func.isRequired,
  getBlackboxFacsimiles: _react.PropTypes.func.isRequired,
  setMapStateToProps: _react.PropTypes.func.isRequired,
  unsetMapStateToProps: _react.PropTypes.func.isRequired
};