'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports["default"] = connect;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _storeShape = require('../utils/storeShape');

var _storeShape2 = _interopRequireDefault(_storeShape);

var _shallowEqual = require('../utils/shallowEqual');

var _shallowEqual2 = _interopRequireDefault(_shallowEqual);

var _wrapActionCreators = require('../utils/wrapActionCreators');

var _wrapActionCreators2 = _interopRequireDefault(_wrapActionCreators);

var _warning = require('../utils/warning');

var _warning2 = _interopRequireDefault(_warning);

var _getDisplayName = require('../utils/getDisplayName');

var _getDisplayName2 = _interopRequireDefault(_getDisplayName);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var defaultStateProps = {};

var defaultMapOwnProps = function defaultMapOwnProps(props) {
  var blackbox = props.blackbox;

  var otherProps = _objectWithoutProperties(props, ['blackbox']);

  return otherProps;
};
var defaultMapStateToProps = function defaultMapStateToProps(state) {
  return defaultStateProps;
}; // eslint-disable-line no-unused-vars
var defaultMapDispatchToProps = function defaultMapDispatchToProps(dispatch) {
  return { dispatch: dispatch };
};
var defaultMergeProps = function defaultMergeProps(stateProps, dispatchProps, parentProps) {
  return _extends({}, parentProps, stateProps, dispatchProps);
};

function didPropsChange(newProp, propertyName, component) {
  if ((0, _shallowEqual2["default"])(newProp, component[propertyName] || {})) {
    return false;
  }
  component[propertyName] = newProp;
  return true;
}

var ShouldComponentUpdate = function (_Component) {
  _inherits(ShouldComponentUpdate, _Component);

  function ShouldComponentUpdate() {
    _classCallCheck(this, ShouldComponentUpdate);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  ShouldComponentUpdate.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
    return nextProps.update;
  };

  ShouldComponentUpdate.prototype.render = function render() {
    return this.props.renderElement();
  };

  return ShouldComponentUpdate;
}(_react.Component);

function connect(mapStateToProps, mapDispatchToProps, mergeProps) {
  var options = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

  var mapStateToProps_selector = mapStateToProps || defaultMapStateToProps;
  var mapStateToProps_ownPropsSelector = defaultMapOwnProps;
  var mapStateToProps_definitelyAFactory = false;
  if (Array.isArray(mapStateToProps)) {
    mapStateToProps_selector = mapStateToProps[0];
    mapStateToProps_ownPropsSelector = mapStateToProps[1] || defaultMapOwnProps;
    mapStateToProps_definitelyAFactory = !!mapStateToProps[2];
  }
  var mapDispatchToProps_selector = mapDispatchToProps || defaultMapDispatchToProps;
  var mapDispatchToProps_ownPropsSelector = defaultMapOwnProps;
  var mapDispatchToProps_definitelyAFactory = false;
  if (Array.isArray(mapDispatchToProps)) {
    mapDispatchToProps_selector = mapDispatchToProps[0];
    mapDispatchToProps_ownPropsSelector = mapDispatchToProps[1] || defaultMapOwnProps;
    mapDispatchToProps_definitelyAFactory = !!mapDispatchToProps[2];
  }

  if (typeof mapDispatchToProps_selector !== 'function') {
    mapDispatchToProps_selector = (0, _wrapActionCreators2["default"])(mapDispatchToProps_selector);
  }
  var allOwnPropsSelectors = {
    mapDispatchToProps_ownPropsSelector: mapDispatchToProps_ownPropsSelector,
    mapStateToProps_ownPropsSelector: mapStateToProps_ownPropsSelector
  };
  var finalMergeProps = mergeProps || defaultMergeProps;
  var _options$pure = options.pure;
  var pure = _options$pure === undefined ? true : _options$pure;
  var _options$withRef = options.withRef;
  var withRef = _options$withRef === undefined ? false : _options$withRef;

  var checkMergedEquals = pure && finalMergeProps !== defaultMergeProps;

  return function wrapWithConnect(WrappedComponent) {
    var connectDisplayName = 'Connect(' + (0, _getDisplayName2["default"])(WrappedComponent) + ')';

    function checkStateShape(props, methodName) {
      if (!(0, _isPlainObject2["default"])(props)) {
        (0, _warning2["default"])(methodName + '() in ' + connectDisplayName + ' must return a plain object. ' + ('Instead received ' + props + '.'));
      }
    }

    function checkOwnPropsSelector(selectorFunction, type) {
      if (selectorFunction === defaultMapOwnProps) {
        (0, _warning2["default"])(type + '() in ' + connectDisplayName + ' uses props in the selector as the second argument,' + ' but no selector has been provided for those props. Please consider supplying a props selector' + ' in the following format: connect([ (state, resultProps)=>{...}, (props)=>resultProps ], ...)');
      }
    }

    function computeMergedProps(stateProps, dispatchProps, parentProps) {
      var mergedProps = finalMergeProps(stateProps, dispatchProps, parentProps);
      if (process.env.NODE_ENV !== 'production') {
        checkStateShape(mergedProps, 'mergeProps');
      }
      return mergedProps;
    }

    var Connect = function (_Component2) {
      _inherits(Connect, _Component2);

      function Connect(props, context) {
        _classCallCheck(this, Connect);

        (0, _invariant2["default"])(props.store || context.store, 'Could not find "store" in either the context or ' + ('props of "' + connectDisplayName + '". ') + 'Either wrap the root component in a <Provider>, ' + ('or explicitly pass "store" as a prop to "' + connectDisplayName + '".'));
        (0, _invariant2["default"])(props.blackbox, 'Could not find "blackbox" in the ' + ('props of "' + connectDisplayName + '". '));

        var _this2 = _possibleConstructorReturn(this, _Component2.call(this, props, context));

        _this2.appendMapStateToProps = _this2.appendMapStateToProps.bind(_this2);
        _this2.setWrappedInstance = _this2.setWrappedInstance.bind(_this2);
        _this2.store = _this2.props.store || _this2.context.store;
        _this2.uniqueId = context.getUniqueId();
        _this2.appendStateProps = {
          names: {},
          selectors: {},
          inputs: {}
        };
        _this2.stateProps = {
          appendStateProps: {}
        };
        _this2.mapStateToProps_ownProps = mapStateToProps_ownPropsSelector(_this2.props);
        _this2.lastFilteredBlackbox = {};
        _this2.mapStateToProps_firstRun_isNotFactory = false;

        var testMapStateToProps = void 0;
        if (mapStateToProps_definitelyAFactory) {
          testMapStateToProps = mapStateToProps_selector(_this2.store.getState(), props);
        } else {
          testMapStateToProps = mapStateToProps_selector(_this2.store.getState(), _this2.mapStateToProps_ownProps);
        }
        if (typeof testMapStateToProps === 'function') {
          _this2.mapStateToProps_selector = testMapStateToProps;
        } else {
          _this2.mapStateToProps_firstRun_isNotFactory = true;
          _this2.mapStateToProps_selector = mapStateToProps_selector;
          _this2.stateProps = _this2.cacheAndSetAllStateProps(testMapStateToProps);
        }
        if (_this2.mapStateToProps_selector.length === 1) {
          _this2.mapStateToProps_dependsOnProps = false;
        } else {
          if (process.env.NODE_ENV !== 'production') {
            checkOwnPropsSelector(mapStateToProps_ownPropsSelector, 'mapStateToProps');
          }
          _this2.mapStateToProps_dependsOnProps = true;
        }

        var testMapDispatchToProps = void 0;
        var dispatchFactory = void 0;
        _this2.mapDispatchToProps_ownProps = mapDispatchToProps_ownPropsSelector(_this2.props);
        _this2.updateDispatchEverytime = false;
        if (mapDispatchToProps_definitelyAFactory) {
          testMapDispatchToProps = mapDispatchToProps_selector(_this2.store.dispatch, props);
        } else {
          testMapDispatchToProps = mapDispatchToProps_selector(_this2.store.dispatch, _this2.mapDispatchToProps_ownProps);
        }
        if (typeof testMapDispatchToProps === 'function') {
          dispatchFactory = true;
          _this2.mapDispatchToProps_selector = testMapDispatchToProps;
        } else {
          dispatchFactory = false;
          _this2.mapDispatchToProps_firstRun_isNotFactory = true;
          _this2.mapDispatchToProps_selector = mapDispatchToProps_selector;
        }
        if (_this2.mapDispatchToProps_selector.length === 1) {
          _this2.mapDispatchToProps_dependsOnProps = false;
        } else {
          if (_this2.mapDispatchToProps_selector.length === 0) {
            _this2.updateDispatchEverytime = true;
          }
          if (process.env.NODE_ENV !== 'production') {
            checkOwnPropsSelector(mapStateToProps_ownPropsSelector, 'mapStateToProps');
          }
          _this2.mapDispatchToProps_dependsOnProps = true;
        }
        _this2.dispatchProps = dispatchFactory ? _this2.mapDispatchToProps_selector(_this2.store.dispatch, _this2.mapDispatchToProps_ownProps) : testMapDispatchToProps;
        if (process.env.NODE_ENV !== 'production') {
          checkStateShape(_this2.dispatchProps, 'mapDispatchToProps');
        }
        return _this2;
      }

      Connect.prototype.componentDidMount = function componentDidMount() {
        this.context.addConnectIds && this.context.addConnectIds([this.uniqueId]);
      };

      Connect.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps) {
        var nextBlackbox = nextProps.blackbox;

        var nextOwnProps = _objectWithoutProperties(nextProps, ['blackbox']);

        var _props = this.props;
        var blackbox = _props.blackbox;

        var ownProps = _objectWithoutProperties(_props, ['blackbox']);

        var blackboxChanged = false;
        if (!(0, _shallowEqual2["default"])(nextOwnProps, ownProps)) {
          this.haveOwnPropsChanged = true;
        } else {
          this.haveOwnPropsChanged = false;
        }
        if (!pure) {
          return true;
        }
        if (!(0, _shallowEqual2["default"])(blackbox, nextBlackbox)) {
          blackboxChanged = true;
        }
        return blackboxChanged || this.haveOwnPropsChanged;
      };

      Connect.prototype.componentWillUnmount = function componentWillUnmount() {
        this.context.unsetMapStateToProps(this.uniqueId);
        this.context.removeConnectIds && this.context.removeConnectIds([this.uniqueId]);
        this.clearCache();
      };

      Connect.prototype.clearCache = function clearCache() {
        this.store = null;
        this.uniqueId = null;
        this.mapStateToProps_selector = null;
        this.mapStateToProps_ownProps = null;
        this.mapDispatchToProps_selector = null;
        this.mapDispatchToProps_ownProps = null;
        this.lastFilteredBlackbox = null;
        this.lastFilteredBlackbox_ownProps = null;
        this.stateProps = null;
        this.dispatchProps = null;
        this.mergedProps = null;
        this.renderedElement = null;
      };

      Connect.prototype.getWrappedInstance = function getWrappedInstance() {
        (0, _invariant2["default"])(withRef, 'To access the wrapped instance, you need to specify ' + '{ withRef: true } as the fourth argument of the connect() call.');
        return this.wrappedInstance;
      };

      Connect.prototype.getDispatchProps = function getDispatchProps() {
        if (!this.mapDispatchToProps_firstRun_isNotFactory && !this.updateDispatchEverytime && !this.mapDispatchToProps_ownProps_didChange()) {
          return [this.dispatchProps, false];
        }
        return [this.computeDispatchProps(), true];
      };

      Connect.prototype.getStateProps = function getStateProps() {
        var blackbox = this.props.blackbox;

        var stateProps = blackbox[this.uniqueId];
        if (stateProps && !this.mapStateToProps_ownProps_didChange()) {
          return stateProps;
        }
        return this.computeStateProps();
      };

      Connect.prototype.genericProps_ownProps_didChange = function genericProps_ownProps_didChange(name) {
        var ownPropsName = 'map' + name + 'ToProps_ownProps';
        var _props2 = this.props;
        var blackbox = _props2.blackbox;

        var ownProps = _objectWithoutProperties(_props2, ['blackbox']);

        var nextOwnProps = allOwnPropsSelectors['map' + name + 'ToProps_ownPropsSelector'](ownProps);
        if ((0, _shallowEqual2["default"])(this[ownPropsName], nextOwnProps)) {
          return false;
        }
        this[ownPropsName] = nextOwnProps;
        return true;
      };

      Connect.prototype.mapStateToProps_ownProps_didChange = function mapStateToProps_ownProps_didChange() {
        if (this.mapStateToProps_dependsOnProps) {
          return this.genericProps_ownProps_didChange('State');
        }
        return false;
      };

      Connect.prototype.mapDispatchToProps_ownProps_didChange = function mapDispatchToProps_ownProps_didChange() {
        if (this.mapDispatchToProps_dependsOnProps) {
          return this.genericProps_ownProps_didChange('Dispatch');
        }
        return false;
      };

      Connect.prototype.computeStateProps = function computeStateProps() {
        if (this.mapStateToProps_firstRun_isNotFactory) {
          this.mapStateToProps_firstRun_isNotFactory = false;
          return this.stateProps;
        }
        var stateProps = this.mapStateToProps_selector(this.store.getState(),
        //Maybe use proxy object here to throw error when calling undefined properties
        mapStateToProps_ownPropsSelector(this.props));
        return this.cacheAndSetAllStateProps(stateProps);
      };

      Connect.prototype.cacheAndSetAllStateProps = function cacheAndSetAllStateProps(stateProps) {
        this.context.setMapStateToProps(this.uniqueId, this.allMapStateToProps_selector());
        this.statePropsBase = stateProps;
        if (process.env.NODE_ENV !== 'production') {
          checkStateShape(stateProps, 'mapStateToProps');
        }
        return _extends({}, stateProps, { appendStateProps: this.stateProps.appendStateProps });
      };

      Connect.prototype.computeDispatchProps = function computeDispatchProps() {
        if (this.mapDispatchToProps_firstRun_isNotFactory) {
          this.mapDispatchToProps_firstRun_isNotFactory = false;
          return this.dispatchProps;
        }
        var dispatchProps = this.mapDispatchToProps_selector(this.store.dispatch,
        //Maybe use proxy object here to throw error when calling undefined properties
        this.mapDispatchToProps_ownProps);
        if (process.env.NODE_ENV !== 'production') {
          checkStateShape(dispatchProps, 'mapDispatchToProps');
        }
        return dispatchProps;
      };

      Connect.prototype.filterBlackbox_ownProps = function filterBlackbox_ownProps() {
        var _props3 = this.props;
        var blackbox = _props3.blackbox;

        var ownProps = _objectWithoutProperties(_props3, ['blackbox']);

        var nextBlackbox = _extends({}, blackbox);
        delete nextBlackbox[this.uniqueId];
        if (didPropsChange(nextBlackbox, 'lastFilteredBlackbox', this) || this.haveOwnPropsChanged !== false) {
          this.lastFilteredBlackbox_ownProps = _extends({ blackbox: this.lastFilteredBlackbox }, ownProps);
        }
        return this.lastFilteredBlackbox_ownProps || {};
      };

      Connect.prototype.updateMergedPropsIfNeeded = function updateMergedPropsIfNeeded(stateProps, dispatchProps, ownProps) {
        var nextMergedProps = computeMergedProps(stateProps, dispatchProps, ownProps);
        if (this.mergedProps && checkMergedEquals && (0, _shallowEqual2["default"])(nextMergedProps, this.mergedProps || {})) {
          return false;
        }
        this.mergedProps = nextMergedProps;
        return true;
      };

      Connect.prototype.inputsChanged = function inputsChanged(name, inputs) {
        return !(0, _shallowEqual2["default"])(this.appendStateProps.inputs[name] || {}, inputs);
      };

      Connect.prototype.appendStateProps_nameExists_inBlackbox = function appendStateProps_nameExists_inBlackbox(name) {
        return this.props.blackbox[this.uniqueId] && this.props.blackbox[this.uniqueId].appendStateProps && this.props.blackbox[this.uniqueId].appendStateProps[name];
      };

      Connect.prototype.computeAppendStateProps = function computeAppendStateProps(name, inputs, selector) {
        this.setAppendStateProps(name, inputs, selector);
        this.context.setMapStateToProps(this.uniqueId, this.allMapStateToProps_selector());
        var thisAppendStateProps = selector(this.store.getState(), inputs);
        this.stateProps.appendStateProps[name] = thisAppendStateProps;
        return thisAppendStateProps;
      };

      Connect.prototype.appendMapStateToProps = function appendMapStateToProps(name, inputs, selector) {
        if (!this.inputsChanged(name, inputs) && this.appendStateProps_nameExists_inBlackbox(name)) {
          return this.props.blackbox[this.uniqueId].appendStateProps[name];
        }
        return this.computeAppendStateProps(name, inputs, selector);
      };

      Connect.prototype.setAppendStateProps = function setAppendStateProps(name, inputs, selector) {
        this.appendStateProps.names[name] = null;
        this.appendStateProps.selectors[name] = selector;
        this.appendStateProps.inputs[name] = inputs;
      };

      Connect.prototype.allMapStateToProps_selector = function allMapStateToProps_selector() {
        var _this3 = this;

        return function (state) {
          var changed = false;
          var appendStatePropsChanged = false;
          var baseMapStateToProps = _this3.mapStateToProps_selector(state, _this3.mapStateToProps_ownProps);
          if (_this3.statePropsBase !== baseMapStateToProps) {
            changed = true;
            _this3.statePropsBase = baseMapStateToProps;
            _this3.stateProps = _extends({ appendStateProps: _this3.stateProps.appendStateProps }, baseMapStateToProps);
          }
          Object.keys(_this3.appendStateProps.names).forEach(function (name) {
            var thisAppendStateProps = _this3.appendStateProps.selectors[name](state, _this3.appendStateProps.inputs[name]);
            if (_this3.stateProps.appendStateProps[name] !== thisAppendStateProps) {
              _this3.stateProps.appendStateProps[name] = thisAppendStateProps;
              if (!appendStatePropsChanged) {
                appendStatePropsChanged = true;
                _this3.stateProps.appendStateProps = _extends({}, _this3.stateProps.appendStateProps);
              }
            }
          });
          if (appendStatePropsChanged || changed) {
            _this3.statePropsChanged = true;
          } else {
            _this3.statePropsChanged = false;
          }
          if (appendStatePropsChanged) {
            return [appendStatePropsChanged, _extends({}, _this3.stateProps)];
          }
          return [changed, _this3.stateProps];
        };
      };

      Connect.prototype.setWrappedInstance = function setWrappedInstance(ref) {
        this.wrappedInstance = ref;
      };

      Connect.prototype.render = function render() {
        var _this4 = this;

        var nextStateProps = this.getStateProps();

        var _getDispatchProps = this.getDispatchProps();

        var nextDispatchProps = _getDispatchProps[0];
        var knowForSureTheSameProps = _getDispatchProps[1];

        var haveStatePropsChanged = this.statePropsChanged || didPropsChange(nextStateProps, 'stateProps', this);
        var haveDispatchPropsChanged = knowForSureTheSameProps || didPropsChange(nextDispatchProps, 'dispatchProps', this);
        var haveMergedPropsChanged = false;
        if (haveStatePropsChanged || haveDispatchPropsChanged || this.haveOwnPropsChanged !== false) {
          haveMergedPropsChanged = this.updateMergedPropsIfNeeded(nextStateProps, nextDispatchProps, this.filterBlackbox_ownProps());
        }
        if (!haveMergedPropsChanged && pure && this.renderedElement) {
          return _react2["default"].createElement(ShouldComponentUpdate, {
            update: false,
            key: 'connect-' + this.uniqueId
          });
        }
        if (withRef) {
          this.renderedElement = _react2["default"].createElement(ShouldComponentUpdate, {
            update: true,
            key: 'connect-' + this.uniqueId,
            renderElement: function renderElement() {
              return (0, _react.createElement)(WrappedComponent, _extends({}, _this4.mergedProps, {
                ref: _this4.setWrappedInstance,
                appendMapStateToProps: _this4.appendMapStateToProps
              }));
            }
          });
        } else {
          this.renderedElement = _react2["default"].createElement(ShouldComponentUpdate, {
            update: true,
            key: 'connect-' + this.uniqueId,
            renderElement: function renderElement() {
              return (0, _react.createElement)(WrappedComponent, _extends({}, _this4.mergedProps, { appendMapStateToProps: _this4.appendMapStateToProps }));
            }
          });
        }
        return this.renderedElement;
      };

      return Connect;
    }(_react.Component);

    Connect.displayName = connectDisplayName;
    Connect.WrappedComponent = WrappedComponent;
    Connect.contextTypes = {
      store: _storeShape2["default"],
      getUniqueId: _react.PropTypes.func.isRequired,
      setMapStateToProps: _react.PropTypes.func.isRequired,
      unsetMapStateToProps: _react.PropTypes.func.isRequired,
      addConnectIds: _react.PropTypes.func
    };
    Connect.propTypes = {
      store: _storeShape2["default"]
    };

    return (0, _hoistNonReactStatics2["default"])(Connect, WrappedComponent);
  };
}